#!/bin/bash
# vim: set tabstop=8 shiftwidth=4 softtabstop=4 expandtab smarttab colorcolumn=80:
#
# Copyright (c) 2016 Red Hat, Inc.
# Author: Nathaniel McCallum <npmccallum@redhat.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

is_bound_to_tang() {
    local dev
    for dev in $(lsblk -p -n -s -r \
                 | awk '$6 == "crypt" { getline; print $1 }' | sort -u); do
       if clevis luks list -d "${dev}" 2>/dev/null | grep -q tang; then
           return 0
       fi
    done
    return 1
}

configure_passwd_watchers() {
    if ! command -v systemctl >/dev/null; then
        return 1
    fi

    local proc_cmdline
    proc_cmdline=$(</proc/cmdline)

    local luks_uuid
    local cfg
    for dev in $(lsblk -p -n -s -r \
                 | awk '$6 == "crypt" { getline; print $1 }' | sort -u); do
        luks_uuid=$(cryptsetup luksUUID "${dev}")
        [[ "${proc_cmdline}" == *"rd.luks.uuid=luks-${luks_uuid}"* ]] && continue

        if cfg=$(clevis luks list -d "${dev}" 2>/dev/null); then
            local action=enable
            [ -z "${cfg}" ] && action=disable
            systemctl "${action}" "clevis-luks-askpass@${luks_uuid}.path" 2>/dev/null
        fi
    done
}

depends() {
    local depends="crypt systemd"
    if is_bound_to_tang; then
        depends=$(printf "%s network" "${depends}")
    fi
    echo "${depends}"
}

tang_cmdline() {
    echo "rd.neednet=1"
}

install() {
    local ret=0

    if is_bound_to_tang; then
        tang_cmdline > "${initdir}/etc/cmdline.d/99clevis.conf"
    fi

    inst_hook initqueue/online 60 "$moddir/clevis-hook.sh"
    inst_hook initqueue/settled 60 "$moddir/clevis-hook.sh"

    inst_multiple /etc/services \
        clevis-decrypt-tang \
        clevis-decrypt-sss \
        /usr/libexec/clevis-luks-askpass \
        clevis-decrypt \
        cryptsetup \
        luksmeta \
        clevis \
        mktemp \
        curl \
        jose \
        nc

    for cmd in clevis-decrypt-tpm2 \
	tpm2_createprimary \
	tpm2_pcrlist \
	tpm2_unseal \
	tpm2_load; do

	if ! find_binary "$cmd" &>/dev/null; then
	    ((ret++))
	fi
    done

    if (($ret == 0)); then
	inst_multiple clevis-decrypt-tpm2 \
	    tpm2_createprimary \
	    tpm2_pcrlist \
	    tpm2_unseal \
	    tpm2_load
	inst_libdir_file "libtss2-tcti-device.so*"
    fi

    configure_passwd_watchers
    dracut_need_initqueue
}

installkernel() {
    hostonly='' instmods =drivers/char/tpm
}
